#!/usr/bin/env perl

use strict;
use warnings;
use feature ":5.10";

use JSON;
use Moose::Util qw/find_meta does_role search_class_by_role/;
use Module::Find;

my @event_classes = useall state51::Logger::Event;

my %classes;
foreach my $class (sort @event_classes) {
	next if is_role($class);
	$classes{$class} = class_as_hash($class);
}

say "{";
foreach my $class (keys %classes) {
	#	say encode_json( $classes{$class} );
	say to_json( $classes{$class}, { utf8 => 1, pretty => 1 } );
	say ",";
}
say "}";

# sjson schema but in a hash
sub class_as_hash {
    my ($class) = @_;

    #warn is_role($class);

    my $meta = find_meta($class);

    my %attributes;
    foreach my $attribute ($meta->get_all_attributes) {
        $attributes{$attribute->name} = {type => 'string'};
    }

    my %result;
    $result{title}      = $class;
    $result{type}       = 'object';
    $result{properties} = \%attributes;

    return \%result;
}


sub is_role {
    my $package_or_obj = shift;

    my $meta = find_meta($package_or_obj);
    return if not $meta;
    return $meta->isa('Moose::Meta::Role');
}
