#!/usr/bin/env perl
use strict;
use warnings;
use feature ':5.10';

use Module::Find;

use Net::AMQP::RabbitMQ;
use Try::Tiny;
use Time::HiRes qw(gettimeofday);
use JSON;

my %event_classes;
my @event_classes = useall state51::Logger::Event;
@event_classes{@event_classes} = undef;

my $host     = $ENV{MQHOST} || die "need ENV MQHOST";
my $exchange = 'firehose';
my $channel  = 1;

my $queue_name = 'firehose_validator';

my $mq = Net::AMQP::RabbitMQ->new();

my $rec = 0;
my %seen;

RESTART:
eval {
    $mq->connect(
        $host => {
            user        => $ENV{MQUSER},
            password    => $ENV{MQPASS},
            heartbeat   => 3600,
            channel_max => 1,
        }
    );
};
if ($@) {
    warn localtime() . "wonky";
    die $@;
}

eval { $mq->channel_open($channel); };
if ($@) {
    warn localtime() . "wonky";
    die $@;
}

$mq->basic_qos($channel, {prefetch_count => 200, global => 0});

my ($queuename, $waiting_count, $consumers) = $mq->queue_declare(
    $channel,
    $queue_name,
    {
        auto_delete => 0,
        durable     => 1,
    }
);

$mq->consume(
    $channel,
    $queue_name,
    {
        no_ack       => 0,
        consumer_tag => "worker_$$",
        exclusive    => 0
    }
);

while (1) {
    $mq->heartbeat();
    my $payload;
    try {
        $payload = $mq->recv(15)
          ; # blocks until message/timeout (no heartbeats when no messages, sadface)
    }
    catch {
        warn "recv caught error: $_";
        goto RESTART;
    };

    # should be below but "Unexpected header: 1!"
    if (not defined $payload) {
        #warn "undef payload!";
        sleep(1);
        goto RESTART;
    }

    #print localtime() . " Worker $$: Received from queue $queuename\n";
    $rec++;

    my $body = $payload->{body};
    my $dtag = $payload->{delivery_tag};
    my $routing_key = $payload->{routing_key};

    if ($routing_key =~ m/reindex_service/) {
        my $message_hash = from_json($body, {utf8 => 0});

        my $eventname = $message_hash->{EventName};

        if ($eventname eq 'Generic') {
            say $message_hash->{date}." ".$message_hash->{pid}." ".$message_hash->{message};
        } elsif ($eventname eq 'MySQLReplicationLag') {
            say $message_hash->{date}." ".$message_hash->{pid}." MySQL Lag ".$message_hash->{lag}." seconds";
        } elsif ($eventname eq 'LockClaimed') {
            say $message_hash->{date}." ".$message_hash->{pid}." Claim Lock ".$message_hash->{name};
        } elsif ($eventname eq 'LockReleased') {
            say $message_hash->{date}." ".$message_hash->{pid}." Release Lock ".$message_hash->{name};
        } elsif ($eventname eq 'ScriptStart') {
            say $message_hash->{date}." ".$message_hash->{pid}." Script Start ".$message_hash->{name};
        } else {
            use DDP;
            p $message_hash;
        }
    }

    $mq->ack($channel, $dtag);
}
