package state51::Logger::Event::MailoutRun;
use Moose;
use MooseX::Types::Moose qw/ Int Str /;
use state51::Types qw/InstanceID/;
use MooseX::ClassAttribute;
use namespace::clean -except => 'meta';

extends 'state51::Logger::Event';

with 'state51::Logger::Role::WithRetailer';

class_has documentation => (
    is      => 'ro',
    isa     => Str,
    default => 'This event is sent during a mailout run, once per role mailed. You should see as many of these for a mailout as there are mailable subscribers.',
);

has [qw/ subject list_name /] => (
    isa => Str,
    is => 'ro',
    required => 1,
);

has user => (
    isa => Str,
    is => 'ro',
    default => sub {
        my ($uid) = getpwuid($<);
        return $uid;
    },
);

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{Mailout "%s" running for mailing list "%s" by %s},
    attributes => [qw/ subject list_name user /],
};

has total => (
    isa => Int,
    is => 'ro',
    required => 1,
);

has run => (
    isa => Int,
    is => 'ro',
    required => 1,
);

has mailout_id => (
    isa => InstanceID,
    is => 'ro',
    required => 1,
);

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{Mailout "%s" running for mailing list "%s" by %s},
    attributes => [qw/ subject list_name user /],
};

__PACKAGE__->meta->make_immutable;
1;

