package state51::Logger::Event::Task;
use Moose;
use namespace::autoclean;
use MooseX::ClassAttribute;

extends 'state51::Logger::Event';

with 'state51::Logger::Event::Role::JobExecution';

class_has documentation => (
    is => 'ro',
    isa => 'Str',
    default => 'These task events are raised by airflow. Current events include task completion and failure.',
);

has 'task_id' => (
    is => 'rw',
    isa => 'Str',
    required => 1,
    documentation => 'The task id. Roughly describes a specific task living within a given DAG workflow, e.g. `ingest`',
);

has 'dag_id' => (
    is => 'rw',
    isa => 'Str',
    required => 1,
    documentation => 'The dag id. Roughly describes the aim(s) of the job as a whole, e.g. `deezer_dailies_to_es`',
);

has 'run_id' => (
    is => 'rw',
    isa => 'Str',
    required => 1,
    documentation => 'The run id. Provides the logical date, prefixed by the run type (manual/scheduled), e.g. `scheduled__2021-11-28T05:00:00+00:00`',
);

has 'logical_date' => (
    is => 'rw',
    isa => 'Str',
    required => 1,
    documentation => 'The logical date. The datetime that we have ran the DAG for, rather than (necessarily) the date at runtime, e.g. `2021-11-28T05:00:00+00:00`',
);

with 'state51::Log::Message::Structured::Stringify::Sprintf' => {
    format_string => q{Task %s started},
    attributes => [qw/ task_id /],
};

__PACKAGE__->meta->make_immutable;

